#include <QGuiApplication>
#include <QRect>
#include <QWindow>
#include <QColor>
#include <QFontMetrics>
#include <QExposeEvent>
#include <QPainter>
#include <QBackingStore>
#include <QStringList>
#include <QSharedPointer>
#include <QScreen>
#include <QDebug>

class Window : public QWindow
{
public:
    Window(QScreen *screen, const QColor color);

public slots:
    void render();

protected:
    void mousePressEvent(QMouseEvent *);
    void exposeEvent(QExposeEvent *);
    void resizeEvent(QResizeEvent *);

private:
    const QColor m_color;
    QBackingStore *m_backingStore;
};

Window::Window(QScreen *screen, const QColor color)
    : QWindow(screen), m_color(color)
    , m_backingStore(new QBackingStore(this))
{
    setFlags(flags() | Qt::WindowTitleHint | Qt::WindowSystemMenuHint
             | Qt::WindowMinMaxButtonsHint | Qt::WindowCloseButtonHint);
}

void Window::render()
{
    enum { spacing = 4 };
    const QRect geomRect = QRect(QPoint(0, 0), geometry().size());
    m_backingStore->resize(geomRect.size());
    m_backingStore->beginPaint(geomRect);

    QPaintDevice *device = m_backingStore->paintDevice();

    QPainter p(device);
    p.fillRect(geomRect, m_color);
    const QString line1 = title();
    const QString line2 = "Left click: Toggle fullscreen, middle click: Quit";
    const QFontMetrics metrics(p.font());
    QRect textRect = metrics.boundingRect(line2);

    QPoint textPoint = geomRect.center() - QPoint(textRect.width() / 2, textRect.height() + spacing /2 );
    p.drawText(textPoint, line1);
    textPoint.ry() += spacing + textRect.height();
    p.drawText(textPoint, line2);
    m_backingStore->endPaint();    
    m_backingStore->flush(geomRect, this);
}

void Window::exposeEvent(QExposeEvent *e)
{
    // qDebug() << __FUNCTION__ << this << e->region().boundingRect();
    if (!e->region().isEmpty())
        render();
}

void Window::resizeEvent(QResizeEvent *)
{
    if (isExposed())
        render();
}

void Window::mousePressEvent(QMouseEvent *m)
{
    switch (m->button()) {
    case Qt::LeftButton: {
        const bool isFullScreen = windowState() == Qt::WindowFullScreen;
        qDebug() << this << "Toggling fullscreen " << (!isFullScreen) << screen();
        setWindowState(isFullScreen ? Qt::WindowNoState : Qt::WindowFullScreen);
    }
        break;
    case Qt::RightButton:
        break;
    case Qt::MiddleButton:
        qDebug() << "Quit";
        qApp->quit();
        break;
    default:
        break;
    }
}

int main(int argc, char **argv)
{
    typedef QSharedPointer<QWindow> WindowPtr;
    typedef QList<WindowPtr> WindowList;

    QGuiApplication app(argc, argv);
    const QStringList args = QCoreApplication::arguments();

    WindowList windows;

    QList<QScreen*> screenList = app.screens();
    qDebug() << screenList.size() << "screens";

    for (int i= 0; i < screenList.size(); ++i) {
        QScreen *screen =  screenList.at(i);
        const Qt::GlobalColor color = (Qt::GlobalColor)(Qt::red + i);
        qDebug() << "Creating window on #" << i << screen->name();
        WindowPtr w(new Window(screen, color));
        w->setObjectName(screen->name() + QStringLiteral("-Window"));
        w->setTitle(QString::fromLatin1("%1 %2 [%3]")
                    .arg(QT_VERSION_STR, screen->name(), qApp->platformName()));
        w->resize(700, 200);
        if (args.contains("-normal")) {
            if (!args.contains("-noposition"))
                w->setFramePosition(QPoint(10, 10));
            w->show();
        } else {
            w->showFullScreen();
        }
        windows.push_back(w);
    }
    return app.exec();
}
